/*----------------------Web Analytics-----------------------------------------*/
/* weba_visitors                                                                   */
/*                                                                            */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* Supported By: Colin Smith                                                  */
/*                                                                            */
/*----------------------------------------------------------------------------*/
/* History:                                                                   */
/* Date          Description                    Username         Change Code  */
/* 20090504   Unicode compatibility             cabahl           S0576264     */
/* 20090722   Changed storcnt logic             sascos                        */
/* 20120227   Changed formats to informats      sascos                        */
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* Copyright (c) 2005-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved                  */
/*----------------------------------------------------------------------------*/

/* <!
  $Revision: 1.1.2.4.8.1 $
  $Date: 2012/02/28 16:14:33 $
-->
*/

%macro weba_visitors ();

%if %sysfunc(exist(&visitor)) and %sysfunc(exist(&report)) %then
%do;
   /* Figure out the storage repository.  RDBMS or SAS? */
   %let stortyp=;         
   proc sql noprint;              
      select engine into :stortyp                
      from dictionary.libnames                   
      where libname = upcase(trim(left("%scan(&report,1,.)")));    

      select count(1) into :storcnt
      from dictionary.libnames                   
      where libname = upcase(trim(left("%scan(&report,1,.)")));             
	  
	  select count(1) into :vdow
	  from sashelp.vcolumn
	  where libname = upcase(trim(left("%scan(&report,1,.)"))) and 
	        memname = upcase(trim(left("%scan(&report,2,.)"))) and
			upcase(name) = 'DAY_OF_WEEK';
   quit;  

   /* do we have day of week on the report set? */
   /* if not, roll up visitor info before join  */
   %if &vdow = 1 %then %let vdow = day_of_week;
   %else %let vdow = ;

   %if %upcase(&stortyp) ne ORACLE and %eval(&storcnt+0) >= 1 %then
   %do;

      /* First make sure that visitor has requisite index if SAS table */
      %weba_check_index(tbl=&visitor);

      /* Break Visitor information down to session_dt level */
      /* Calculate visitor counts simultaneously.     */

      data work.visitors;

         /* Set up and retain the new count fields */
         attrib Repeat_Visitors Unique_Visitors 
                Page_View_low_session_cnt 
                Page_View_med_session_cnt 
                Page_View_high_session_cnt format=8.;
         retain Repeat_Visitors Unique_Visitors 
                Page_View_low_session_cnt
                Page_View_med_session_cnt 
                Page_View_high_session_cnt 0;

         /* Read in the data by session_dt */
         set &VISITOR (keep=session_dt session_count page_view_low_session_cnt 
                            page_view_med_session_cnt page_view_high_session_cnt &vdow
                       rename=(page_view_low_session_cnt=page1 
                               page_view_med_session_cnt=page2
                               page_view_high_session_cnt=page5));
         by session_dt &vdow;

         if first.session_dt
		 %if &vdow=day_of_week %then or first.day_of_week;
		 then do; /* Initialize fields */
            unique_visitors=0;
            repeat_visitors=0;
            page_view_low_session_cnt=0;
            page_view_med_session_cnt=0;
            page_view_high_session_cnt=0;
         end; /* Initialize fields */

         /* business rule: a repeat visitor has more than one */
         /* session_count              */
         /* otherwise it is a unique visitor                  */

         if session_count gt 1 then repeat_visitors=repeat_visitors+1;
         unique_visitors=unique_visitors+1;

         page_view_low_session_cnt=page_view_low_session_cnt+page1;
         page_view_med_session_cnt=page_view_med_session_cnt+page2;
         page_view_high_session_cnt=page_view_high_session_cnt+page5;

         /* retain one record per session_dt. */
         if last.session_dt 
		 %if &vdow=day_of_week %then or last.day_of_week; 
		 then output;

      run;

      /* Create two formats for easier merging later      */
      /* by creating two data sets and a format from each */

      data work.uniques (drop=session_dt);
         attrib start format=$200.;
		 retain fmtname 'Uniques' type 'I';
         set work.visitors (rename=(unique_visitors=label) 
                            drop=repeat_visitors Page_View_low_session_cnt 
                                 Page_View_med_session_cnt Page_View_high_session_cnt) end=last;
		 %if &vdow=day_of_week %then start=trim(left(input(put(session_dt,date9.),$20.)))!!trim(left(&vdow ));
		 %else start=trim(left(input(put(session_dt,date9.),$20.)));
         ; output;
      run;

      proc format cntlin=work.uniques;
      run;

      data work.repeats (drop=session_dt);
         attrib start format=$200.;
		 retain fmtname 'Repeats' type 'I';
         set work.visitors (rename=(repeat_visitors=label) 
                            drop=unique_visitors Page_View_low_session_cnt 
                                 Page_View_med_session_cnt Page_View_high_session_cnt) end=last;
         %if &vdow=day_of_week %then start=trim(left(input(put(session_dt,date9.),$20.)))!!trim(left(&vdow ));
		 %else start=trim(left(input(put(session_dt,date9.),$20.)));
		 ; output;
      run;

      proc format cntlin=work.repeats;
      run;   

      /* Now do the same thing for the page views */

      data work.under1 (drop=session_dt);
         attrib start format=$200.;
		 retain fmtname 'Pagea' type 'I';
         set work.visitors (rename=(page_view_low_session_cnt=label) 
                            drop=unique_visitors repeat_visitors 
                                  page_view_med_session_cnt page_view_high_session_cnt) end=last;
         %if &vdow=day_of_week %then start=trim(left(input(put(session_dt,date9.),$20.)))!!trim(left(&vdow ));
		 %else start=trim(left(input(put(session_dt,date9.),$20.)));
		 ; output;
      run;

      proc format cntlin=work.under1;
      run;   

      data work.twoup (drop=session_dt);
         attrib start format=$200.;
		 retain fmtname 'Pageb' type 'I';
         set work.visitors (rename=(page_view_med_session_cnt=label) 
                            drop=unique_visitors repeat_visitors 
                                 page_view_low_session_cnt page_view_high_session_cnt) end=last;
         %if &vdow=day_of_week %then start=trim(left(input(put(session_dt,date9.),$20.)))!!trim(left(&vdow ));
		 %else start=trim(left(input(put(session_dt,date9.),$20.)));
  	     ; output;
      run;

      proc format cntlin=work.twoup;
      run;   

      data work.fiveup (drop=session_dt);
	     attrib start format=$200.;
         retain fmtname 'Pagec' type 'I';
         set work.visitors (rename=(page_view_high_session_cnt=label) 
                            drop=unique_visitors repeat_visitors 
                                 page_view_low_session_cnt page_view_med_session_cnt) end=last;
         %if &vdow=day_of_week %then start=trim(left(input(put(session_dt,date9.),$20.)))!!trim(left(&vdow ));
		 %else start=trim(left(input(put(session_dt,date9.),$20.)));
		 ; output;
      run;

      proc format cntlin=work.fiveup;
      run;   

      /* Add the visitor information via format */
      data &report (drop=st);
	     attrib st format=$200.;
         attrib unique_visitors repeat_visitors format=16.;
         attrib page_view_med_session_cnt
                page_view_low_session_cnt 
                page_view_high_session_cnt format=16.; 
         set &report (drop=unique_visitors repeat_visitors);
		 %if &vdow=day_of_week %then st=trim(left(input(put(session_dt,date9.),$20.)))!!trim(left(&vdow ));
		 %else st=trim(left(input(put(session_dt,date9.),$20.)));
		 ;
         unique_visitors=input(st,uniques.);
         repeat_visitors=input(st,repeats.);
         page_view_low_session_cnt=input(st,pagea.);
         page_view_med_session_cnt=input(st,pageb.);
         page_view_high_session_cnt=input(st,pagec.);
      run;

      /* Add the index back */
      %weba_check_index(tbl=&report);

   %end; /* SAS Datasets */

   %else %if %eval(&storcnt+0) >= 1 %then %do; /* Oracle */

      /* Figure out which of the 5 fields are already on the report */
      %let cnt=0;

      %let vr1=UNIQUE_VISITORS;
      %let vr2=REPEAT_VISITORS;
      %let vr3=PAGE_VIEW_LOW_SESSION_CNT;
      %let vr4=PAGE_VIEW_MED_SESSION_CNT;
      %let vr5=PAGE_VIEW_HIGH_SESSION_CNT;

      data _null_;
         set sashelp.vcolumn (where=(libname=ktrim(kleft(upcase("%kscan(&report,1,.)"))) and
              upcase(memname)=ktrim(kleft(upcase("%kscan(&report,2,.)"))) and
              upcase(ktrim(kleft(name))) in ('UNIQUE_VISITORS',
              'REPEAT_VISITORS','PAGE_VIEW_LOW_SESSION_CNT',
              'PAGE_VIEW_MED_SESSION_CNT','PAGE_VIEW_HIGH_SESSION_CNT')));
         if upcase(ktrim(kleft(name)))='UNIQUE_VISITORS' then call symput('vr1','');
         else if upcase(ktrim(kleft(name)))='REPEAT_VISITORS' then call symput('vr2','');
         else if upcase(ktrim(kleft(name)))='PAGE_VIEW_LOW_SESSION_CNT' then call symput('vr3','');
         else if upcase(ktrim(kleft(name)))='PAGE_VIEW_MED_SESSION_CNT' then call symput('vr4','');
         else if upcase(ktrim(kleft(name)))='PAGE_VIEW_HIGH_SESSION_CNT' then call symput('vr5','');
      run;

      /* Add the ones that aren't */

      %do x=1 %to 5; /* Add the columns */

         %if %length(%ktrim(%kleft(%bquote(&&vr&x)))) gt 1 %then
         %do; /* add the requisite field */

            proc sql noprint;
            &ora_pass_thru_connect         
            execute (ALTER TABLE %kscan(&report,2,.)
               ADD &&vr&x NUMERIC
            ) by Oracle; disconnect from Oracle; quit;
			
         %end; /* add the requisite field */
      %end; /* Add the columns */

      proc sql noprint;
         &ora_pass_thru_connect         
         execute (			   
            update %kscan(&report,2,.) t
            set (unique_visitors, repeat_visitors, page_view_low_session_cnt, 
                 page_view_med_session_cnt, page_view_high_session_cnt) =
                (select count(1),
                        sum(case when session_count > 1 then 1 else 0 end),
                        sum(v.page_view_low_session_cnt),  
                        sum(v.page_view_med_session_cnt),
                        sum(v.page_view_high_session_cnt)
                from %kscan(&visitor,2,.) v
                where t.session_dt = v.session_dt 
				%if &vdow=day_of_week %then and t.day_of_week = v.day_of_week;
                group by session_dt))             
         by Oracle;   
         disconnect from Oracle;                 
      quit;

   %end; /* Oracle */
%end;
%mend weba_visitors;
